from collections import deque, defaultdict

def find_guarded_warehouses(n, m, k, roads, guards):
    # Шаг 1: Построение графа
    graph = defaultdict(list)
    for a, b in roads:
        graph[a].append(b)
        graph[b].append(a)

    # Шаг 2: Множество для хранения охраняемых складов
    guarded_warehouses = set()

    # Шаг 3: Обход для каждого охранника
    for p, h in guards:
        # BFS
        queue = deque([(p, 0)])  # (номер склада, расстояние)
        visited = set()
        visited.add(p)

        while queue:
            current_warehouse, distance = queue.popleft()
            if distance <= h:
                guarded_warehouses.add(current_warehouse)
                for neighbor in graph[current_warehouse]:
                    if neighbor not in visited:
                        visited.add(neighbor)
                        queue.append((neighbor, distance + 1))

    # Шаг 4: Подготовка результата
    guarded_warehouses = sorted(guarded_warehouses)
    return len(guarded_warehouses), guarded_warehouses

# Пример использования
n, m, k = map(int, input().split())
roads = [tuple(map(int, input().split())) for _ in range(m)]
guards = [tuple(map(int, input().split())) for _ in range(k)]

count, warehouses = find_guarded_warehouses(n, m, k, roads, guards)

# Вывод результата
print(count)
if count > 0:
    print(" ".join(map(str, warehouses)))
else:
    print()