from collections import deque, defaultdict

def find_guarded_warehouses(n, m, k, roads, guards):
    # Построение графа
    graph = defaultdict(list)
    for a, b in roads:
        graph[a].append(b)
        graph[b].append(a)

    # Множество для хранения охраняемых складов
    guarded_warehouses = set()

    # Обработка каждого охранника
    for p, h in guards:
        # BFS для нахождения охраняемых складов
        queue = deque([(p, 0)])  # (текущий склад, текущее расстояние)
        visited = set()
        visited.add(p)

        while queue:
            current_warehouse, current_distance = queue.popleft()
            # Добавление текущего склада в охраняемые
            guarded_warehouses.add(current_warehouse)

            # Если достигли максимального расстояния, не продолжаем
            if current_distance < h:
                for neighbor in graph[current_warehouse]:
                    if neighbor not in visited:
                        visited.add(neighbor)
                        queue.append((neighbor, current_distance + 1))

    # Подготовка результата
    guarded_warehouses = sorted(guarded_warehouses)
    return len(guarded_warehouses), guarded_warehouses

# Чтение входных данных
n, m, k = map(int, input().split())
roads = [tuple(map(int, input().split())) for _ in range(m)]
guards = [tuple(map(int, input().split())) for _ in range(k)]

# Получение результата
count, guarded_warehouses = find_guarded_warehouses(n, m, k, roads, guards)

# Вывод результата
print(count)
if count > 0:
    print(" ".join(map(str, guarded_warehouses)))